<?php

App::uses('AppShell', 'Console/Command');

App::uses('AclShell', 'Console/Command');
App::uses('AclExtrasShell', 'AclExtras.Console/Command');

class PermissionsShell extends AppShell {

	protected $_Aro;

	protected $_Aco;

	protected $_Permission;

	public function getOptionParser() {
		$parser = parent::getOptionParser();
		$parser->addOption(
			'update-default-permissions',
			[
				'short' => 'u',
				'help' => 'Once synchronised, update the default permissions for the default user groups',
				'boolean' => true
			]
		);
		$parser->addOption(
			'skip-sync',
			[
				'short' => 's',
				'help' => 'Skip the synchronisation step.',
				'boolean' => true
			]
		);

		return $parser;
	}

	public function startup() {
		parent::startup();

		$this->_Aro = EvClassRegistry::init('Aro');
		$this->_Aco = EvClassRegistry::init('Aco');
		$this->_Permission = EvClassRegistry::init('Permission');
	}

	public function sync() {
		if (empty($this->params['skip-sync'])) {
			//Pull in all the controller actions to be used as aco records.
			$aclExtrasShell = new AclExtrasShell();
			$aclExtrasShell->startup();
			$aclExtrasShell->aco_sync();
		}

		if (!empty($this->params['update-default-permissions'])) {
			$model = 'UserGroup';
			$foriegnKey = 1;

			$this->out("Updating default permissions for " . $model . ' ' . $foriegnKey);
			$this->_allowGlobalPermission($model, $foriegnKey);

			$model = 'UserGroup';
			$foriegnKey = 2;

			$this->out("Updating default permissions for " . $model . ' ' . $foriegnKey);
			$this->_allowGlobalPermission($model, $foriegnKey);

			$model = 'UserGroup';
			$foriegnKey = 3;

			$this->out("Updating default permissions for " . $model . ' ' . $foriegnKey);
			$this->_denyAdminPermission($model, $foriegnKey);
			$this->_allowGlobalPermission($model, $foriegnKey);
		}

		return null;
	}

	protected function _allowGlobalPermission($aroModel, $aroKey) {
		$globalAcos = $this->_Aco->find(
			'all',
			[
				'conditions' => [
					'Aco.parent_id' => null,
					'Aco.alias' => 'controllers',
				]
			]
		);

		$acoIds = Hash::extract($globalAcos, '{n}.Aco.id');

		if (!empty($acoIds)) {
			return $this->_allowPermissions($acoIds, $aroModel, $aroKey);
		}

		return false;
	}

	protected function _denyAdminPermission($aroModel, $aroKey) {
		$adminAcos = $this->_Aco->find(
			'all',
			[
				'conditions' => [
					'Aco.alias LIKE' => 'admin_%',
				]
			]
		);

		$acoIds = Hash::extract($adminAcos, '{n}.Aco.id');

		if (!empty($acoIds)) {
			return $this->_denyPermissions($acoIds, $aroModel, $aroKey);
		}

		return false;
	}

	protected function _allowPermissions($acoIds, $aroModel, $aroKey) {
		$currentAro = $this->_getAroFromModelKey($aroModel, $aroKey);

		if (!empty($currentAro['Aro']['id'])) {
			$permissions = [];
			foreach ($acoIds as $acoId) {
				$permissions[] = $this->_allow($currentAro['Aro']['id'], $acoId);
			}

			if (count($permissions) == count($acoIds)) {
				return true;
			} else {
				return false;
			}
		}

		return null;
	}

	protected function _denyPermissions($acoIds, $aroModel, $aroKey) {
		$currentAro = $this->_getAroFromModelKey($aroModel, $aroKey);

		if (!empty($currentAro['Aro']['id'])) {
			$permissions = [];
			foreach ($acoIds as $acoId) {
				$result = $this->_allow($currentAro['Aro']['id'], $acoId, '*', -1);
				$permissions[] = $result;
			}

			if (count($permissions) == count($acoIds)) {
				return true;
			} else {
				return false;
			}
		}

		return null;
	}

	protected function _getAroFromModelKey($aroModel, $aroKey) {
		return $this->_Aro->find(
			'first',
			[
				'conditions' => [
					'Aro.model' => $aroModel,
					'Aro.foreign_key' => $aroKey,
				]
			]
		);
	}

	protected function _allow($aroId, $acoId, $actions = '*', $permission = 1) {
		$savePermission = [
			'aro_id' => $aroId,
			'aco_id' => $acoId,
		];

		$currentPermission = $this->_Permission->find(
			'first',
			[
				'conditions' => [
					'Permission.aro_id' => $aroId,
					'Permission.aco_id' => $acoId,
				]
			]
		);

		if (!empty($currentPermission)) {
			$savePermission['id'] = $currentPermission['Permission']['id'];
		}

		$permissionActions = $this->_Permission->getAcoKeys($this->_Permission->schema());

		foreach ($permissionActions as $permissionAction) {
			if (!empty($currentPermission) && isset($currentPermission['Permission'][$permissionAction])) {
				$savePermission[$permissionAction] = $currentPermission['Permission'][$permissionAction];
			}

			if (is_array($actions)) {
				foreach ($actions as $action) {
					if (in_array($action, $permissionActions)) {
						$savePermission[$action] = $permission;
					}
				}
			} elseif (is_string($actions) && $actions != '*') {
				if (in_array($actions, $permissionActions)) {
					$savePermission[$actions] = $permission;
				}
			} elseif ($actions == '*') {
				foreach ($permissionActions as $permissionAction) {
					$savePermission[$permissionAction] = $permission;
				}
			}
		}

		$this->_Permission->create();
		return $this->_Permission->save($savePermission);
	}
}
